package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkNewService;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.ExcelUtils;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.enterprise.api.service.StoreWidgetService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.ChannelCodeEnum;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.StaffEditJsonQO;
import com.gic.haoban.manage.web.vo.ClerkInfoVo;
import com.gic.haoban.manage.web.vo.StaffExportVO;
import com.gic.haoban.manage.web.vo.StaffVO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;


@RestController
public class StaffController extends WebBaseController {
    private static final Logger logger = LoggerFactory.getLogger(StaffController.class);

    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private WxApplicationApiService wxApplicationApiService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private ClerkNewService clerkNewService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;
    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
    @Autowired
    private ClerkMainStoreRelatedApiService clerkMainStoreRelatedApiService;
    @Autowired
    private HaobanUserApiService haobanUserApiService;
    @Autowired
    private DealSyncOperationApiService dealSyncOperationApiService;
    @Autowired
    private StoreWidgetService storeWidgetService;


    @RequestMapping("set-default-gicenterprise")
    public HaobanResponse updateDefaultGicEid(String enterpriseId) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String userId = login.getHanbaoUserId();
        this.haobanUserApiService.updateDefaultGicEid(userId, enterpriseId);
        return this.success(null);
    }

    @RequestMapping("staff-sel")
    public HaobanResponse staffSel(String staffId) {
        StaffDTO staff = staffApiService.selectById(staffId);
        List<StaffDepartmentRelatedDTO> list = staffApiService.listStaffDepartmentByStaffId(staffId);
        List<DepartmentDTO> departmentList = new ArrayList<DepartmentDTO>();
        for (StaffDepartmentRelatedDTO staffDepartmentRelatedDTO : list) {
            DepartmentDTO dto = departmentApiService.selectById(staffDepartmentRelatedDTO.getDepartmentId());
            departmentList.add(dto);

        }
        staff.setDepartmentList(departmentList);
        return resultResponse(HaoBanErrCode.ERR_1, staff);
    }

    /**
     * 分页查询员工列表
     *
     * @param departmentId 部门id
     * @param activeFlag   激活状态 - 后期删除
     * @param keyWord      关键字
     * @param relationFlag 关联状态
     * @param pageInfo     分页参数
     * @return res
     */
    @RequestMapping("staff-list")
    public HaobanResponse staffList(String departmentId,
                                    Integer activeFlag,
                                    Integer relationFlag,
                                    String keyWord,
                                    BasePageInfo pageInfo) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<String> departmentIds = new ArrayList<>();
        DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
        if (departmentDTO != null) {
            String parentDepartmentId = departmentDTO.getParentDepartmentId();
            if (!"0".equals(parentDepartmentId)) {
                // 部门
                departmentIds = departmentApiService.listSonDeptIdsByDepartmentId(departmentId, wxEnterpriseId);
            }
        }
        //员工
        Page<StaffDTO> page = staffApiService.pageStaff(departmentIds, activeFlag, keyWord, pageInfo, relationFlag, wxEnterpriseId);

        logger.info("pages={},totalPage={}", page.getCurrentPage(), page.getTotalPage());

        return resultResponse(HaoBanErrCode.ERR_1, PageUtil.getPageInfo(page));
    }

    @RequestMapping("staff-edit")
    public HaobanResponse staffEdit(StaffDTO staffDTO, String departmentIds) {
        String staffId = staffDTO.getStaffId();
        StaffDTO staff = staffApiService.selectById(staffId);
        if (staff == null) {
            return resultResponse(HaoBanErrCode.ERR_10006);
        }
        String phoneNumber = staffDTO.getPhoneNumber();
        String nationcode = staffDTO.getNationCode();
        String wxEnterpriseId = staff.getWxEnterpriseId();
        StaffDTO old = staffApiService.selectByNationcodeAndPhoneNumber(wxEnterpriseId, nationcode, phoneNumber);
        if (old != null && !old.getStaffId().equals(staffId)) {
            return resultResponse(HaoBanErrCode.ERR_10005);
        }
        staffApiService.staffEdit(staffDTO, departmentIds);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("staff-invite")
    public HaobanResponse staffInvite(String staffIds) {
        if (StringUtils.isBlank(staffIds)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        String[] staffArr = staffIds.split(",");
        List<String> userArr = new ArrayList<>();
        String wxEnterpriseId = "";
        for (String staffId : staffArr) {
            StaffDTO staff = staffApiService.selectById(staffId);
            if (staff != null) {
                String userId = staff.getWxUserId();
                wxEnterpriseId = staff.getWxEnterpriseId();
                WxEnterpriseQwDTO qwDTO = wxEnterpriseApiService.getQwInfo(wxEnterpriseId);
                if (qwDTO.needOpenUserId3th()) {
                    userId = staff.getWxOpenUseId();
                }
                userArr.add(userId);
            }
        }
        WxApplicationDTO application = wxApplicationApiService.selectByWxEnterpriseIdAndApplicationType(wxEnterpriseId, 1);
        if (application != null) {
            qywxUserApiService.inviteUser(application.getCorpid(), application.getSiteId(), userArr);
        }
        return resultResponse(HaoBanErrCode.ERR_1);

    }

    @RequestMapping("staff-batch-edit-department")
    public HaobanResponse staffBatchEditDepartment(String params) {
        List<StaffEditJsonQO> list = JSON.parseArray(params, StaffEditJsonQO.class);
        for (StaffEditJsonQO staffEditJsonQO : list) {
            String staffDepartmentRelatedId = staffEditJsonQO.getStaffDepartmentRelatedId();
            String departmentIds = staffEditJsonQO.getCurrDepartmentIds();
            StaffDepartmentRelatedDTO related = staffDepartmentRelatedApiService.getByStaffDepartmentRelatedId(staffDepartmentRelatedId);
            if (related != null) {
                StaffDTO staff = staffApiService.selectById(related.getStaffId());
                if (staff != null) {
                    staffApiService.staffEdit(staff, departmentIds);
                }
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }


    @RequestMapping("get-clerk-info")
    public HaobanResponse getClerkInfo(String clerkId) {
        StaffVO staffVO = new StaffVO();
        ClerkDTO clerk = clerkService.getClerkByClerkId(clerkId);
        if (clerk != null) {
            String storeId = clerk.getStoreId();
            StoreDTO storeDTO = storeService.getStore(storeId);
            staffVO.setStaffName(clerk.getClerkName());
            staffVO.setNationCode(clerk.getNationcode());
            staffVO.setPhoneNumber(clerk.getPhoneNumber());
            staffVO.setPosition(clerk.getPositionName());
            staffVO.setHeadImg(clerk.getHeadImgUrl());
            staffVO.setDepartmentName(storeDTO == null ? "" : storeDTO.getStoreName());
        }
        return resultResponse(HaoBanErrCode.ERR_1, staffVO);
    }

    @RequestMapping("get-staff-info")
    public HaobanResponse getStaffInfo(String staffId) {
        StaffVO staffVO = new StaffVO();
        StaffDTO staff = staffApiService.selectById(staffId);
        if (staff != null) {
            List<StaffDepartmentRelatedDTO> list = staffDepartmentRelatedApiService.listByStaffId(staffId);
            staffVO.setStaffName(staff.getStaffName());
            staffVO.setNationCode(staff.getNationCode());
            staffVO.setPhoneNumber(staff.getPhoneNumber());
            staffVO.setPosition(staff.getPostion());
            staffVO.setHeadImg(staff.getHeadImg());
            if (CollectionUtil.isNotEmpty(list)) {
                String departmentId = list.get(0).getDepartmentId();
                DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
                if (departmentDTO != null) {
                    staffVO.setDepartmentName(departmentDTO.getDepartmentName());
                }
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, staffVO);
    }

    /**
     * 成员关联关系接口
     *
     * @param staffId
     * @return
     */
    @RequestMapping("staff-clerk-relation")
    public HaobanResponse staffClerkList(String staffId) {
        StaffDTO staff = staffApiService.selectById(staffId);
        if (staff == null) {
            return resultResponse(HaoBanErrCode.ERR_10007);
        }
        List<StaffClerkInfoDTO> retList = staffClerkRelationApiService.listBindDetailByStaffId(staffId);
        if (CollectionUtils.isEmpty(retList)) {
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        ClerkMainStoreRelatedDTO mainStoreRelatedDTO = clerkMainStoreRelatedApiService.getWxEnterpriseIdAndStaffId(staff.getWxEnterpriseId(), staffId);
        for (StaffClerkInfoDTO dto : retList) {
            if (mainStoreRelatedDTO != null) {
                String storeId = dto.getStoreId();
                if (StringUtils.isNotBlank(storeId) && storeId.equals(mainStoreRelatedDTO.getStoreId())) {
                    dto.setMainStoreFlag(1);
                } else {
                    dto.setMainStoreFlag(0);
                }
            }
            if (dto.getManageFlag() != null && dto.getManageFlag() == 1) {
                dto.setManageStoreCount(staffApiService.getCountHaoBanStoreIdsByClerkId(dto.getClerkId(), staff.getWxEnterpriseId()).getStoreCount());
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, retList);
    }

    /**
     * 成员关联关系接口
     *
     * @param clerkId
     * @return
     */
    @RequestMapping("del-clerk-relation")
    public HaobanResponse delStaffClerkList(String clerkId) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        boolean b = staffClerkRelationApiService.unbindByStaffAndClerkId(login.getClerkId(), clerkId, ChannelCodeEnum.ADMIN_UNBIND.getCode());
        return resultResponse(HaoBanErrCode.ERR_1, b);
    }

    /**
     * 成员关联关系搜索接口
     *
     * @param search
     * @return
     */
    @RequestMapping("search-clerk-relation")
    public HaobanResponse staffClerkSearch(String search, Integer manageFlag) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        String enterpriseId = login.getEnterpriseId();
        if (null == wxEnterpriseId) {
            return resultResponse(HaoBanErrCode.ERR_4);
        }
        List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtil.isEmpty(list)) {
            return resultResponse(HaoBanErrCode.ERR_10010);
        }
        List<String> enterpriseIds = list.stream().filter(dto -> {
            boolean over = wxEnterpriseApiService.enterpriseIsOver(dto.getEnterpriseId());
            return !over;
        }).map(EnterpriseDetailDTO::getEnterpriseId).collect(Collectors.toList());

        List<String> storeIds = wxEnterpriseRelatedApiService.listStoreIdByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtils.isEmpty(storeIds)) {
            logger.info("没有权限门店");
            return resultResponse(HaoBanErrCode.ERR_1);
        }
        logger.info("好办门店数={},wxEnterpriseId={},enterpriseIds={}", storeIds.size(), wxEnterpriseId, enterpriseIds);
        List<ClerkDTO> clerkDTOList = new ArrayList<>();
        if (manageFlag != null && manageFlag == 1) {
            clerkDTOList = clerkService.listClerkEnableHaoban(enterpriseIds, search);
        } else {
            clerkDTOList = clerkNewService.listClerkByEnterpriseIdAndSearchAndClerkType(enterpriseIds, storeIds, search);
        }
        List<ClerkInfoVo> retList = EntityUtil.changeEntityListByJSON(ClerkInfoVo.class, clerkDTOList);
        if (CollectionUtils.isNotEmpty(retList)) {
            logger.info("导购数量={}", retList.size());
            List<String> clerkIds = retList.stream().map(ClerkInfoVo::getClerkId).collect(Collectors.toList());
            List<StaffClerkRelationDTO> dtos = new ArrayList<>();
            if (manageFlag != null && manageFlag == 1) {
                dtos = staffClerkRelationApiService.listByClerkIdsWxEnterpriseId(clerkIds, wxEnterpriseId);
            } else {
                dtos = staffClerkRelationApiService.listByClerkIds(clerkIds);
            }
            if (CollectionUtils.isNotEmpty(dtos)) {
                Map<String, StaffClerkRelationDTO> hasRelationClerkIds = dtos.stream().collect(Collectors.toMap(StaffClerkRelationDTO::getClerkId, mid -> mid));
                retList.forEach(clerkInfoVo -> {
                    if (hasRelationClerkIds.containsKey(clerkInfoVo.getClerkId())) {
                        clerkInfoVo.setRelationStatus(1);
                        StaffDTO staffDTO = staffApiService.selectById(hasRelationClerkIds.get(clerkInfoVo.getClerkId()).getStaffId());
                        if (staffDTO != null) {
                            clerkInfoVo.setStaffName(staffDTO.getStaffName());
                            clerkInfoVo.setStaffId(staffDTO.getStaffId());
                        }
                    }
                    StoreDTO store = storeService.getStore(clerkInfoVo.getStoreId());
                    if (null != store) {
                        clerkInfoVo.setStoreName(store.getStoreName());

                    }
                });
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1, retList);
    }

    /**
     * 新建关联
     *
     * @param staffId
     * @return
     */
    @RequestMapping("add-clerk-relation")
    public HaobanResponse staffClerkSearch(String staffId, String clerkId, Integer manageFlag) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        if (StringUtils.isAnyBlank(staffId, clerkId)) {
            return resultResponse(HaoBanErrCode.ERR_2);
        }
        StaffDTO staffDTO = staffApiService.selectById(staffId);
        if (null == staffDTO) {
            return resultResponse(HaoBanErrCode.ERR_10007);
        }
        ClerkDTO clerkDTO = clerkService.getClerkByClerkId(clerkId);
        if (null == clerkDTO) {
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        StaffClerkRelationDTO relationDTO = new StaffClerkRelationDTO();
        relationDTO.setClerkCode(clerkDTO.getClerkCode());
        relationDTO.setClerkId(clerkId);
        relationDTO.setStaffId(staffId);
        relationDTO.setEnterpriseId(clerkDTO.getEnterpriseId());
        relationDTO.setStoreId(clerkDTO.getStoreId());
        relationDTO.setWxEnterpriseId(staffDTO.getWxEnterpriseId());
        relationDTO.setManageFlag(manageFlag);

        ServiceResponse response = staffClerkRelationApiService.bindStaffClerk(relationDTO, login.getClerkId(), ChannelCodeEnum.ADMIN_BIND.getCode());
        logger.info("返回信息：{}", JSONObject.toJSONString(response));
        if (response.isSuccess()) {
            return resultResponse(HaoBanErrCode.ERR_1);
        } else {
            HaoBanErrCode errDefine = HaoBanErrCode.ERR_DEFINE;
            errDefine.setMsg(response.getMessage());
            return resultResponse(errDefine);
        }
    }

    /**
     * 绑定日志
     *
     * @return
     */
    @RequestMapping("staff-clerk-bind-log")
    public HaobanResponse bindLogs(String search, @RequestParam(defaultValue = "-1") Integer optType, BasePageInfo qo) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String enterpriseId = login.getEnterpriseId();
        Page<StaffClerkBindLogDetailDTO> page = staffClerkRelationApiService.pageStaffClerkBindLog(login.getWxEnterpriseId(), search, enterpriseId, optType, qo);
        return resultResponse(HaoBanErrCode.ERR_1, page);
    }

    /**
     * 是否展示区经 true是
     * @return
     */
    @RequestMapping("hasManagerFlag")
    public RestResponse hasManagerFlag() {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
        if (CollectionUtil.isEmpty(list)) {
            return RestResponse.failure(Convert.toStr(HaoBanErrCode.ERR_10010.getCode()),HaoBanErrCode.ERR_10010.getMsg());
        }
        List<String> enterpriseIds = list.stream().filter(dto -> {
            boolean over = wxEnterpriseApiService.enterpriseIsOver(dto.getEnterpriseId());
            return !over;
        }).map(EnterpriseDetailDTO::getEnterpriseId).collect(Collectors.toList());
        List<ClerkDTO> clerkDTOList = clerkService.listClerkEnableHaoban(enterpriseIds, null);
        boolean flag = CollectionUtils.isNotEmpty(clerkDTOList);
        return RestResponse.successResult(flag);
    }

    /**
     * @return 返回值类型: <pre>
     * @author 作者: qwm
     * @date 时间: 2020年12月18日 上午11:11:35 <pre>
     * @Description 会员导出
     */
    @RequestMapping("staff-export")
    public HaobanResponse staffExport(HttpServletRequest request, HttpServletResponse response, String departmentId, Integer activeFlag, Integer relationFlag, String keyWord, BasePageInfo pageInfo) {
        List<String> departmentIdSet = new ArrayList<>();
        departmentIdSet.add(departmentId);
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        List<DepartmentDTO> relationList = departmentApiService.listSonByDepartmentIds(departmentIdSet, wxEnterpriseId);
        Map<String, DepartmentDTO> departmentMap = relationList.stream().collect(Collectors.toMap(DepartmentDTO::getDepartmentId, s -> s));
        List<String> departmentIds = relationList.stream().map(DepartmentDTO::getDepartmentId).collect(Collectors.toList());
        List<StaffDTO> staffAllList = Lists.newArrayListWithCapacity(5000);
        pageInfo.setPageSize(Integer.MAX_VALUE);
        int pageNum = 1;
        while (true) {
            pageInfo.setPageNum(pageNum);
            Page<StaffDTO> page = staffApiService.pageStaff(departmentIds, activeFlag, keyWord, pageInfo, relationFlag, wxEnterpriseId);
            List<StaffDTO> list = page.getResult();
            if (CollectionUtils.isEmpty(list)) {
                break;
            }
            staffAllList.addAll(list);
            pageNum++;
        }
        List<StaffExportVO> voList = new ArrayList<>();
        for (StaffDTO staffDTO : staffAllList) {
            StaffExportVO vo = new StaffExportVO();
            vo.setStaffId(staffDTO.getStaffId());
            vo.setPhoneNumber(staffDTO.getPhoneNumber());
            vo.setStaffName(staffDTO.getStaffName());
            vo.setPosition(staffDTO.getPostion());
            vo.setDepartmentName(departmentMap.get(staffDTO.getDepartmentIds()) == null ? null : departmentMap.get(staffDTO.getDepartmentIds()).getDepartmentName());
            vo.setRelationFlag((Objects.isNull(staffDTO.getRelationFlag()) || staffDTO.getRelationFlag() == 0) ? "未关联" : "已关联");
            voList.add(vo);
        }
        //拼接导购信息
        WxEnterpriseDTO enterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
        String wxEnterpriseName = "";
        if (enterprise != null) {
            wxEnterpriseName = enterprise.getCorpName();
        }
        mergeClerkInfo(voList, wxEnterpriseId);
        String fileName = wxEnterpriseName + "通讯录" + DateUtil.dateToStr(new Date(), DateUtil.FORMAT_DATE_8);
        List<String> titleList = Arrays.asList("姓名", "职务", "部门", "手机号", "关联状态", "关联导购");
        List<String> fileList = Arrays.asList("staffName", "position", "departmentName", "phoneNumber", "relationFlag", "clerkName");
        try {
            ExcelUtils.xls(response, request, fileName, voList, fileList, titleList);
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("导出失败");
        }
        return null;
    }

    private void mergeClerkInfo(List<StaffExportVO> list, String wxEnterpriseId) {
        BasePageInfo pageInfoRelation = new BasePageInfo();
        int pageSizeRelation = 1000;
        int pageNumRelation = 1;
        pageInfoRelation.setPageNum(pageNumRelation);
        pageInfoRelation.setPageSize(pageSizeRelation);
        Page<StaffClerkRelationDTO> bindPage = staffClerkRelationApiService.pageByWxEnterpriseId(wxEnterpriseId, pageInfoRelation);
        List<StaffClerkRelationDTO> bindList = bindPage.getResult();
        Set<String> clerkIdList = bindList.stream().map(StaffClerkRelationDTO::getClerkId).collect(Collectors.toSet());
        List<ClerkDTO> clerkList = clerkService.getClerkByClerkIds(clerkIdList);
        while (bindPage.getCurrentPage() < bindPage.getTotalPage()) {
            pageNumRelation++;
            pageInfoRelation.setPageNum(pageNumRelation);
            bindPage = staffClerkRelationApiService.pageByWxEnterpriseId(wxEnterpriseId, pageInfoRelation);
            List<StaffClerkRelationDTO> sonList = bindPage.getResult();
            Set<String> clerkSonIdList = sonList.stream().map(StaffClerkRelationDTO::getClerkId).collect(Collectors.toSet());
            List<ClerkDTO> sonClerkList = clerkService.getClerkByClerkIds(clerkSonIdList);
            if (CollectionUtils.isNotEmpty(sonClerkList)) {
                clerkList.addAll(sonClerkList);
            }
            if (CollectionUtils.isNotEmpty(sonList)) {
                bindList.addAll(sonList);
            }
        }

        Map<String, List<StaffClerkRelationDTO>> bindMap = bindList.stream().collect(Collectors.groupingBy(StaffClerkRelationDTO::getStaffId));
        Map<String, ClerkDTO> clerkMap = clerkList.stream().collect(Collectors.toMap(ClerkDTO::getClerkId, s -> s, (v1, v2) -> v2));
        for (StaffExportVO staffDTO : list) {
            List<StaffClerkRelationDTO> relationList = bindMap.get(staffDTO.getStaffId());
            if (CollectionUtils.isNotEmpty(relationList)) {
                StringBuffer clerkName = new StringBuffer();
                for (StaffClerkRelationDTO staffClerkRelationDTO : relationList) {
                    ClerkDTO clerk = clerkMap.get(staffClerkRelationDTO.getClerkId());
                    if (clerk != null) {
                        clerkName.append(clerk.getClerkName()).append(",");
                    }
                }
                if (StringUtils.isNoneBlank(clerkName.toString())) {
                    clerkName = new StringBuffer(clerkName.substring(0, clerkName.length() - 1));
                }
                staffDTO.setClerkName(clerkName.toString());
            }
        }
    }


    /**
     * 通讯录同步
     *
     * @return
     */
    @RequestMapping("sync-qywx")
    public Object syncQywx() {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        String wxEnterpriseId = login.getWxEnterpriseId();
        WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getOne(wxEnterpriseId);
        if (enterpriseDTO != null) {
            String taskId = dealSyncOperationApiService.createQywxTask(wxEnterpriseId, "后门同步企业微信架构", login.getClerkId(), "后门同步企业微信架构");
            if (StringUtils.isBlank(taskId)) {
                return resultResponse(HaoBanErrCode.ERR_0, false);
            }
            String ret = dealSyncOperationApiService.dealQywxDepartment(taskId, wxEnterpriseId);
            if (null != ret) {
                return resultResponse(HaoBanErrCode.ERR_0, ret, null, null);
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }
}
